@@ -1,125 +1,20 @@
+use crate::expression::flat_details::{self, FlatNodeVec, FlatOpVec};
 use crate::{
-    definitions::N_NODES_ON_STACK,
     expression::{
-        deep::{DeepEx, DeepNode, ExprIdxVec},
+        deep::{DeepBuf, DeepEx, ExprIdxVec},
         partial_derivatives,
     },
-    operators::{self, BinOp, UnaryOp},
+    operators,
     parser::ExParseError,
 };
 use num::Float;
-use smallvec::{smallvec, SmallVec};
 use std::fmt::{self, Debug, Display, Formatter};
-pub type FlatNodeVec<T> = SmallVec<[FlatNode<T>; N_NODES_ON_STACK]>;
-pub type FlatOpVec<T> = SmallVec<[FlatOp<T>; N_NODES_ON_STACK]>;
-
-/// A `FlatOp` contains besides a binary operation an optional unary operation that
-/// will be executed after the binary operation in case of its existence.
-#[derive(Clone, Eq, PartialEq, Ord, PartialOrd, Debug)]
-pub struct FlatOp<T: Copy> {
-    unary_op: UnaryOp<T>,
-    bin_op: BinOp<T>,
-}
-
-#[derive(Clone, Eq, PartialEq, Ord, PartialOrd, Debug)]
-pub enum FlatNodeKind<T: Copy> {
-    Num(T),
-    Var(usize),
-}
-
-#[derive(Clone, Eq, PartialEq, Ord, PartialOrd, Debug)]
-pub struct FlatNode<T: Copy> {
-    kind: FlatNodeKind<T>,
-    unary_op: UnaryOp<T>,
-}
-
-impl<T: Copy> FlatNode<T> {
-    pub fn from_kind(kind: FlatNodeKind<T>) -> FlatNode<T> {
-        FlatNode {
-            kind,
-            unary_op: UnaryOp::new(),
-        }
-    }
-}
-
-fn flatten_vecs<T: Copy + Debug>(
-    deep_expr: &DeepEx<T>,
-    prio_offset: i32,
-) -> (FlatNodeVec<T>, FlatOpVec<T>) {
-    let mut flat_nodes = FlatNodeVec::<T>::new();
-    let mut flat_ops = FlatOpVec::<T>::new();
-
-    for (node_idx, node) in deep_expr.nodes().iter().enumerate() {
-        match node {
-            DeepNode::Num(num) => {
-                let flat_node = FlatNode::from_kind(FlatNodeKind::Num(*num));
-                flat_nodes.push(flat_node);
-            }
-            DeepNode::Var((idx, _)) => {
-                let flat_node = FlatNode::from_kind(FlatNodeKind::Var(*idx));
-                flat_nodes.push(flat_node);
-            }
-            DeepNode::Expr(e) => {
-                let (mut sub_nodes, mut sub_ops) = flatten_vecs(e, prio_offset + 100i32);
-                flat_nodes.append(&mut sub_nodes);
-                flat_ops.append(&mut sub_ops);
-            }
-        };
-        if node_idx < deep_expr.bin_ops().ops.len() {
-            let prio_adapted_bin_op = BinOp {
-                apply: deep_expr.bin_ops().ops[node_idx].apply,
-                prio: deep_expr.bin_ops().ops[node_idx].prio + prio_offset,
-            };
-            flat_ops.push(FlatOp {
-                bin_op: prio_adapted_bin_op,
-                unary_op: UnaryOp::new(),
-            });
-        }
-    }
-
-    if deep_expr.unary_op().op.len() > 0 {
-        if !flat_ops.is_empty() {
-            // find the last binary operator with the lowest priority of this expression,
-            // since this will be executed as the last one
-            let low_prio_op = match flat_ops.iter_mut().rev().min_by_key(|op| op.bin_op.prio) {
-                None => panic!("cannot have more than one flat node but no binary ops"),
-                Some(x) => x,
-            };
-            low_prio_op
-                .unary_op
-                .append_front(&mut deep_expr.unary_op().op.clone());
-        } else {
-            flat_nodes[0]
-                .unary_op
-                .append_front(&mut deep_expr.unary_op().op.clone());
-        }
-    }
-    (flat_nodes, flat_ops)
-}
-
-fn prioritized_indices_flat<T: Copy>(ops: &[FlatOp<T>], nodes: &FlatNodeVec<T>) -> ExprIdxVec {
-    let prio_increase =
-        |bin_op_idx: usize| match (&nodes[bin_op_idx].kind, &nodes[bin_op_idx + 1].kind) {
-            (FlatNodeKind::Num(_), FlatNodeKind::Num(_)) => {
-                let prio_inc = 5;
-                &ops[bin_op_idx].bin_op.prio * 10 + prio_inc
-            }
-            _ => &ops[bin_op_idx].bin_op.prio * 10,
-        };
-    let mut indices: ExprIdxVec = (0..ops.len()).collect();
-    indices.sort_by(|i1, i2| {
-        let prio_i1 = prio_increase(*i1);
-        let prio_i2 = prio_increase(*i2);
-        prio_i2.partial_cmp(&prio_i1).unwrap()
-    });
-    indices
-}
 
 /// Flattens a deep expression
 /// The result does not contain any recursive structures and is faster to evaluate.
 pub fn flatten<T: Copy + Debug>(deepex: DeepEx<T>) -> FlatEx<T> {
-    let (nodes, ops) = flatten_vecs(&deepex, 0);
-    let indices = prioritized_indices_flat(&ops, &nodes);
+    let (nodes, ops) = flat_details::flatten_vecs(&deepex, 0);
+    let indices = flat_details::prioritized_indices_flat(&ops, &nodes);
     let n_unique_vars = deepex.n_vars();
     FlatEx {
         nodes,
@@ -132,9 +27,9 @@ pub fn flatten<T: Copy + Debug>(deepex: DeepEx<T>) -> FlatEx<T> {
 
 /// This is the core data type representing a flattened expression and the result of
 /// parsing a string. We use flattened expressions to make efficient evaluation possible.
-/// Simplified, a flat expression consists of a [`SmallVec`](SmallVec) of nodes and a
-/// [`SmallVec`](SmallVec) of operators that are applied to the nodes in an order following
-/// operator priorities.
+/// Simplified, a flat expression consists of a [`SmallVec`](https://docs.rs/smallvec/) 
+/// of nodes and a [`SmallVec`](https://docs.rs/smallvec/) of operators that are applied
+/// to the nodes in an order following operator priorities.
 ///
 /// You create an expression with the `parse` function or one of its
 /// variants, namely `parse_with_default_ops` and `parse_with_number_pattern`.
@@ -177,7 +72,7 @@ impl<'a, T: Copy + Debug> FlatEx<'a, T> {
     ///            the n-th variable in alphabetical order.
     ///            Thereby, only the first occurrence of the variable in the string is relevant.
     ///            If an expression has been created by partial derivation, the variables always
-    ///            coincide with those of the antiderivatives even in cases where variables are 
+    ///            coincide with those of the antiderivatives even in cases where variables are
     ///            irrelevant such as `(x)'=1`.
     ///
     /// # Errors
@@ -186,45 +81,13 @@ impl<'a, T: Copy + Debug> FlatEx<'a, T> {
     /// the variable slice, we return an [`ExParseError`](ExParseError).
     ///
     pub fn eval(&self, vars: &[T]) -> Result<T, ExParseError> {
-        if self.n_unique_vars != vars.len() {
-            return Err(ExParseError {
-                msg: format!(
-                    "parsed expression contains {} vars but passed slice has {} elements",
-                    self.n_unique_vars,
-                    vars.len()
-                ),
-            });
-        }
-        let mut numbers = self
-            .nodes
-            .iter()
-            .map(|node| {
-                node.unary_op.apply(match node.kind {
-                    FlatNodeKind::Num(n) => n,
-                    FlatNodeKind::Var(idx) => vars[idx],
-                })
-            })
-            .collect::<SmallVec<[T; N_NODES_ON_STACK]>>();
-        let mut ignore: SmallVec<[bool; N_NODES_ON_STACK]> = smallvec![false; self.nodes.len()];
-        for (i, &bin_op_idx) in self.prio_indices.iter().enumerate() {
-            let num_idx = self.prio_indices[i];
-            let mut shift_left = 0usize;
-            while ignore[num_idx - shift_left] {
-                shift_left += 1usize;
-            }
-            let mut shift_right = 1usize;
-            while ignore[num_idx + shift_right] {
-                shift_right += 1usize;
-            }
-            let num_1 = numbers[num_idx - shift_left];
-            let num_2 = numbers[num_idx + shift_right];
-            numbers[num_idx - shift_left] = {
-                let bop_res = (self.ops[bin_op_idx].bin_op.apply)(num_1, num_2);
-                self.ops[bin_op_idx].unary_op.apply(bop_res)
-            };
-            ignore[num_idx + shift_right] = true;
-        }
-        Ok(numbers[0])
+        flat_details::eval_flatex(
+            vars,
+            &self.nodes,
+            &self.ops,
+            &self.prio_indices,
+            self.n_unique_vars,
+        )
     }
 
     /// This method computes a `FlatEx` instance that is a partial derivative of `self` with default operators
@@ -281,7 +144,7 @@ impl<'a, T: Copy + Debug> FlatEx<'a, T> {
     /// Creates an expression string that corresponds to the `FlatEx` instance. This is
     /// not necessarily the input string. More precisely,
     /// * variables are put between curly braces,
-    /// * spaces outside of curly brackets are ignored, 
+    /// * spaces outside of curly brackets are ignored,
     /// * parentheses can be different from the input, and
     /// * expressions are compiled
     /// as shown in the following example.
@@ -307,8 +170,9 @@ impl<'a, T: Copy + Debug> FlatEx<'a, T> {
     }
     /// Usually, a `FlatEx` instance keeps a nested, deep structure of the expression
     /// that is not necessary for evaluation. This functions removes the deep expression
-    /// to reduce memory consumption. [`unparse`](FlatEx::unparse) and the
-    /// [`Display`](std::fmt::Display) implementation will stop working after calling this function.
+    /// to reduce memory consumption. The methods [`partial`](FlatEx::partial) and 
+    /// [`unparse`](FlatEx::unparse) as well as the implementation of the 
+    /// [`Display`](std::fmt::Display) trait will stop working after calling this function.
     pub fn clear_deepex(&mut self) {
         self.deepex = None;
     }
@@ -325,9 +189,87 @@ impl<'a, T: Copy + Debug> Display for FlatEx<'a, T> {
     }
 }
 
+/// This is another representation of a flattened expression besides [`FlatEx`](FlatEx).
+/// The interface is identical. The difference is that [`OwnedFlatEx`](OwnedFlatEx) can be used without
+/// a lifetime parameter. All the data that [`FlatEx`](FlatEx) borrowed is kept in a 
+/// buffer by [`OwnedFlatEx`](OwnedFlatEx). The drawback is that parsing takes longer, since
+/// additional allocations are necessary. Evaluation time should be about the same for 
+/// [`FlatEx`](FlatEx) and [`OwnedFlatEx`](OwnedFlatEx).
+pub struct OwnedFlatEx<T: Copy + Debug> {
+    deepex_buf: Option<DeepBuf<T>>,
+    nodes: FlatNodeVec<T>,
+    ops: FlatOpVec<T>,
+    prio_indices: ExprIdxVec,
+    n_unique_vars: usize,
+}
+impl<T: Copy + Debug> OwnedFlatEx<T> {
+    /// see [`FlatEx::eval`](FlatEx::eval)
+    pub fn eval(&self, vars: &[T]) -> Result<T, ExParseError> {
+        flat_details::eval_flatex(
+            vars,
+            &self.nodes,
+            &self.ops,
+            &self.prio_indices,
+            self.n_unique_vars,
+        )
+    }
+    /// Creates an `OwnedFlatEx` instance from an instance of `FlatEx`.
+    pub fn from_flatex<'a>(flatex: FlatEx<'a, T>) -> Self {
+        Self {
+            deepex_buf: flatex.deepex.map(|d| DeepBuf::from_deepex(&d)),
+            nodes: flatex.nodes,
+            ops: flatex.ops,
+            prio_indices: flatex.prio_indices,
+            n_unique_vars: flatex.n_unique_vars,
+        }
+    }
+    /// See [`FlatEx::partial`](FlatEx::partial).
+    pub fn partial(self, var_idx: usize) -> Result<Self, ExParseError>
+    where
+        T: Float,
+    {
+        let ops = operators::make_default_operators();
+        let deep_buf = match self.deepex_buf {
+            Some(d) => Ok(d),
+            None => Err(ExParseError {
+                msg: "need deep expression for derivation, not possible after calling `clear`"
+                    .to_string(),
+            }),
+        }?;
+        let deepex = deep_buf.to_deepex(&ops)?;
+        let d_i = partial_derivatives::partial_deepex(var_idx, deepex, &ops)?;
+        Ok(Self::from_flatex(flatten(d_i)))
+    }
+    /// See [`FlatEx::unparse`](FlatEx::unparse).
+    pub fn unparse(&self) -> Result<String, ExParseError> {
+        match &self.deepex_buf {
+            Some(deepex) => Ok(deepex.unparsed.clone()),
+            None => Err(ExParseError {
+                msg: "unparse impossible, since deep expression optimized away".to_string(),
+            }),
+        }
+    }
+    /// Clears buffer with owned data, see also [`FlatEx::unparse`](FlatEx::unparse).
+    pub fn clear_deepex(&mut self) {
+        self.deepex_buf = None;
+    }
+}
+/// The expression is displayed as a string created by [`unparse`](OwnedFlatEx::unparse).
+impl<T: Copy + Debug> Display for OwnedFlatEx<T> {
+    fn fmt(&self, f: &mut Formatter) -> fmt::Result {
+        let unparsed = self.unparse();
+        match unparsed {
+            Err(e) => write!(f, "{}", e.msg),
+            Ok(s) => write!(f, "{}", s),
+        }
+    }
+}
+
 #[cfg(test)]
 use crate::{
-    expression::deep::UnaryOpWithReprs, operators::VecOfUnaryFuncs, parse_with_default_ops,
+    expression::{deep::UnaryOpWithReprs, flat_details::FlatNodeKind},
+    operators::{UnaryOp, VecOfUnaryFuncs},
+    parse_with_default_ops,
     util::assert_float_eq_f64,
 };
 
@@ -441,6 +383,15 @@ fn test_display() {
         format!("{}", flatex),
         "unparse impossible, since deep expression optimized away"
     );
+
+    let flatex = flatten(DeepEx::<f64>::from_str("sin(var)/5").unwrap());
+    let mut owned_flatex = OwnedFlatEx::from_flatex(flatex);
+    assert_eq!(format!("{}", owned_flatex), "sin({var})/5.0");
+    owned_flatex.clear_deepex();
+    assert_eq!(
+        format!("{}", owned_flatex),
+        "unparse impossible, since deep expression optimized away"
+    );
 }
 
 #[test]
