@@ -1,4 +1,4 @@
-#![doc(html_root_url = "https://docs.rs/exmex/0.9.2")]
+#![doc(html_root_url = "https://docs.rs/exmex/0.9.3")]
 //! Exmex is a fast, simple, and **ex**tendable **m**athematical **ex**pression evaluator
 //! with the ability to compute [partial derivatives](FlatEx::partial) of expressions.  
 //! ```rust
@@ -27,7 +27,7 @@
 //! #
 //! use exmex;
 //! let to_be_parsed = "log(z) + 2* (-z^2 + sin(4*y))";
-//! let expr = exmex::parse::<f64>(to_be_parsed, &exmex::make_default_operators::<f64>())?;
+//! let expr = exmex::parse_with_default_ops::<f64>(to_be_parsed)?;
 //! assert!((expr.eval(&[3.7, 2.5])? - 14.992794866624788 as f64).abs() < 1e-12);
 //! #
 //! #     Ok(())
@@ -148,6 +148,49 @@
 //! #     Ok(())
 //! # }
 //! ```
+//! 
+//! ## Owned Expression
+//! You cannot return a usual expression from a function without a lifetime parameter, 
+//! since expressions that are instances of [`FlatEx`](FlatEx) keep `&str`s instead of 
+//! `String`s of variable or operator names to make faster parsing possible.
+//! ```rust
+//! # use std::error::Error;
+//! # fn main() -> Result<(), Box<dyn Error>> {
+//! #
+//! use exmex::{self, ExParseError, FlatEx};
+//! fn make<'a>() -> Result<FlatEx::<'a, f64>, ExParseError> {
+//! //            |                        |
+//! //           lifetime parameter necessary
+//!
+//!     let to_be_parsed = "log(z) + 2* (-z^2 + sin(4*y))";
+//!     exmex::parse_with_default_ops::<f64>(to_be_parsed)
+//! }
+//! let expr_owned = make()?;
+//! assert!((expr_owned.eval(&[3.7, 2.5])? - 14.992794866624788 as f64).abs() < 1e-12);
+//! #
+//! #     Ok(())
+//! # }
+//! ``` 
+//! If you are willing to pay the price of roughly doubled parsing times, you can 
+//! obtain an expression that is an instance of [`OwnedFlatEx`](OwnedFlatEx) and owns 
+//! its strings. Evaluation times should be comparable. However, a lifetime parameter is
+//! not needed anymore as shown in the following. 
+//! ```rust
+//! # use std::error::Error;
+//! # fn main() -> Result<(), Box<dyn Error>> {
+//! #
+//! use exmex::{self, ExParseError, OwnedFlatEx};
+//! fn make() -> Result<OwnedFlatEx::<f64>, ExParseError> {
+//!     let to_be_parsed = "log(z) + 2* (-z^2 + sin(4*y))";
+//!     let expr = exmex::parse_with_default_ops::<f64>(to_be_parsed)?;
+//!     Ok(OwnedFlatEx::from_flatex(expr))
+//! }
+//! let expr_owned = make()?;
+//! assert!((expr_owned.eval(&[3.7, 2.5])? - 14.992794866624788 as f64).abs() < 1e-12);
+//! #
+//! #     Ok(())
+//! # }
+//! ```
 //!
 //! ## Priorities and Parentheses
 //! In Exmex-land, unary operators always have higher priority than binary operators, e.g.,
@@ -185,9 +228,12 @@
 //!
 //! ## Serialization and Deserialization
 //!
-//! To use [`serde`](https://serde.rs/) you can activate the feature `serde_support`. 
-//! Currently, this only works for default operators. The implementation 
-//! un-parses and re-parses the whole expression.
+//! To use [`serde`](https://serde.rs/) you can activate the feature `serde_support`.
+//! Currently, this only works for default operators. The implementation
+//! un-parses and re-parses the whole expression. 
+//! [`Deserialize`](https://docs.serde.rs/serde/de/trait.Deserialize.html) and 
+//! [`Serialize`](https://docs.serde.rs/serde/de/trait.Serialize.html) are implemented for
+//! for both, [`FlatEx`](FlatEx) and [`OwnedFlatEx`](OwnedFlatEx).
 //!
 //! ## Unicode
 //! Unicode input strings are currently not supported 😕 but might be added in the
@@ -202,7 +248,7 @@ mod util;
 
 use std::{fmt::Debug, str::FromStr};
 
-pub use expression::flat::FlatEx;
+pub use expression::flat::{FlatEx, OwnedFlatEx};
 use expression::{deep::DeepEx, flat};
 
 use num::Float;
@@ -307,28 +353,28 @@ mod tests {
         operators::{make_default_operators, BinOp, Operator},
         parse, parse_with_default_ops,
         util::{assert_float_eq_f32, assert_float_eq_f64},
-        ExParseError, FlatEx,
+        ExParseError, FlatEx, OwnedFlatEx,
     };
-    
+
     #[test]
     fn test_readme() {
-        fn readme_partial () -> Result<(), ExParseError> {
+        fn readme_partial() -> Result<(), ExParseError> {
             let expr = parse_with_default_ops::<f64>("y*x^2")?;
-            
+
             // d_x
             let dexpr_dx = expr.partial(0)?;
             assert_eq!(format!("{}", dexpr_dx), "({x}*2.0)*{y}");
-            
+
             // d_xy
             let ddexpr_dxy = dexpr_dx.partial(1)?;
             assert_eq!(format!("{}", ddexpr_dxy), "{x}*2.0");
             assert_float_eq_f64(ddexpr_dxy.eval(&[2.0, f64::MAX])?, 4.0);
-            
+
             // d_xyx
             let dddexpr_dxyx = ddexpr_dxy.partial(0)?;
             assert_eq!(format!("{}", dddexpr_dxyx), "2.0");
             assert_float_eq_f64(dddexpr_dxyx.eval(&[f64::MAX, f64::MAX])?, 2.0);
-            
+
             Ok(())
         }
         fn readme() -> Result<(), ExParseError> {
@@ -404,6 +450,7 @@ mod tests {
 
         let sut = "sin(sin(x - 1 / sin(y * 5)) + (5.0 - 1/z))";
         let expr = parse_with_default_ops::<f64>(sut).unwrap();
+        let expr = OwnedFlatEx::from_flatex(expr);
         let reference =
             |x: f64, y: f64, z: f64| ((x - 1.0 / (y * 5.0).sin()).sin() + (5.0 - 1.0 / z)).sin();
         assert_float_eq_f64(
@@ -418,10 +465,12 @@ mod tests {
 
         let sut = "y + 1 + 0.5 * x";
         let expr = parse_with_default_ops::<f64>(sut).unwrap();
+        let expr = OwnedFlatEx::from_flatex(expr);
         assert_float_eq_f64(expr.eval(&[3.0, 1.0]).unwrap(), 3.5);
 
         let sut = " -(-(1+x))";
         let expr = parse_with_default_ops::<f64>(sut).unwrap();
+        let expr = OwnedFlatEx::from_flatex(expr);
         assert_float_eq_f64(expr.eval(&[1.0]).unwrap(), 2.0);
 
         let sut = " sin(cos(-3.14159265358979*x))";
@@ -430,6 +479,7 @@ mod tests {
 
         let sut = "5*sin(x * (4-y^(2-x) * 3 * cos(x-2*(y-1/(y-2*1/cos(sin(x*y))))))*x)";
         let expr = parse_with_default_ops::<f64>(sut).unwrap();
+        let expr = OwnedFlatEx::from_flatex(expr);
         assert_float_eq_f64(expr.eval(&[1.5, 0.2532]).unwrap(), -3.1164569260604176);
 
         let sut = "5*x + 4*y + 3*x";
@@ -438,6 +488,7 @@ mod tests {
 
         let sut = "5*x + 4*y";
         let expr = parse_with_default_ops::<f64>(sut).unwrap();
+        let expr = OwnedFlatEx::from_flatex(expr);
         assert_float_eq_f64(expr.eval(&[0.0, 1.0]).unwrap(), 4.0);
 
         let sut = "5*x + 4*y + x^2";
@@ -539,7 +590,7 @@ mod tests {
                 repr: "None",
                 bin_op: None,
                 unary_op: None,
-            }
+            },
         ];
         let expr = parse("2**2*invert(3)", &custom_ops).unwrap();
         let val = expr.eval(&[]).unwrap();
@@ -564,7 +615,6 @@ mod tests {
         let expr = parse("2^2*1/(berti) + zer0(4)", &extended_operators).unwrap();
         let val = expr.eval(&[4.0]).unwrap();
         assert_float_eq_f32(val, 1.0);
-
     }
 
     #[test]
@@ -573,9 +623,11 @@ mod tests {
             var_idx: usize,
             n_vars: usize,
             random_range: Range<f64>,
-            deri: &FlatEx<f64>,
+            flatex: FlatEx<f64>,
             reference: fn(f64) -> f64,
         ) {
+            let deri = flatex.clone().partial(var_idx).unwrap();
+            println!("flatex {}", flatex);
             println!("partial {}", deri);
             let mut rng = rand::thread_rng();
             for _ in 0..3 {
@@ -585,135 +637,131 @@ mod tests {
                 println!("value under test {}.", vut);
                 assert_float_eq_f64(deri.eval(&vars).unwrap(), reference(vut));
             }
+
+            let owned_flatex = OwnedFlatEx::from_flatex(flatex);
+            println!("flatex owned {}", owned_flatex);
+            let owned_deri = owned_flatex.partial(var_idx).unwrap();
+            println!("partial owned {}", owned_deri);
+            for _ in 0..3 {
+                let vut = rng.gen_range(random_range.clone());
+                let mut vars: SmallVec<[f64; 10]> = smallvec![0.0; n_vars];
+                vars[var_idx] = vut;
+                println!("value under test {}.", vut);
+                assert_float_eq_f64(owned_deri.eval(&vars).unwrap(), reference(vut));
+            }
         }
-        
+
         let sut = "+x";
         println!("{}", sut);
         let var_idx = 0;
         let n_vars = 1;
         let flatex_1 = parse_with_default_ops::<f64>(sut).unwrap();
-        let deri = flatex_1.partial(var_idx).unwrap();
         let reference = |_: f64| 1.0;
-        test(var_idx, n_vars, -10000.0..10000.0, &deri, reference);
+        test(var_idx, n_vars, -10000.0..10000.0, flatex_1, reference);
 
         let sut = "++x";
         println!("{}", sut);
         let var_idx = 0;
         let n_vars = 1;
         let flatex_1 = parse_with_default_ops::<f64>(sut).unwrap();
-        let deri = flatex_1.partial(var_idx).unwrap();
         let reference = |_: f64| 1.0;
-        test(var_idx, n_vars, -10000.0..10000.0, &deri, reference);
+        test(var_idx, n_vars, -10000.0..10000.0, flatex_1, reference);
 
         let sut = "+-+x";
         println!("{}", sut);
         let var_idx = 0;
         let n_vars = 1;
         let flatex_1 = parse_with_default_ops::<f64>(sut).unwrap();
-        let deri = flatex_1.partial(var_idx).unwrap();
         let reference = |_: f64| -1.0;
-        test(var_idx, n_vars, -10000.0..10000.0, &deri, reference);
+        test(var_idx, n_vars, -10000.0..10000.0, flatex_1, reference);
 
         let sut = "-x";
         println!("{}", sut);
         let var_idx = 0;
         let n_vars = 1;
         let flatex_1 = parse_with_default_ops::<f64>(sut).unwrap();
-        let deri = flatex_1.partial(var_idx).unwrap();
         let reference = |_: f64| -1.0;
-        test(var_idx, n_vars, -10000.0..10000.0, &deri, reference);
-
+        test(var_idx, n_vars, -10000.0..10000.0, flatex_1, reference);
 
         let sut = "--x";
         println!("{}", sut);
         let var_idx = 0;
         let n_vars = 1;
         let flatex_1 = parse_with_default_ops::<f64>(sut).unwrap();
-        let deri = flatex_1.partial(var_idx).unwrap();
         let reference = |_: f64| 1.0;
-        test(var_idx, n_vars, -10000.0..10000.0, &deri, reference);
+        test(var_idx, n_vars, -10000.0..10000.0, flatex_1, reference);
 
         let sut = "sin(sin(x))";
         println!("{}", sut);
         let var_idx = 0;
         let n_vars = 1;
         let flatex_1 = parse_with_default_ops::<f64>(sut).unwrap();
-        let deri = flatex_1.partial(var_idx).unwrap();
         let reference = |x: f64| x.sin().cos() * x.cos();
-        test(var_idx, n_vars, -10000.0..10000.0, &deri, reference);
+        test(var_idx, n_vars, -10000.0..10000.0, flatex_1, reference);
 
         let sut = "sin(x)-cos(x)+a";
         println!("{}", sut);
         let var_idx = 1;
         let n_vars = 2;
         let flatex_1 = parse_with_default_ops::<f64>(sut).unwrap();
-        let deri = flatex_1.partial(var_idx).unwrap();
         let reference = |x: f64| x.cos() + x.sin();
-        test(var_idx, n_vars, -10000.0..10000.0, &deri, reference);
-        println!("{}", deri);
-        let deri = deri.partial(var_idx).unwrap();
+        test(var_idx, n_vars, -10000.0..10000.0, flatex_1.clone(), reference);
+        let deri = flatex_1.partial(var_idx).unwrap();
         let reference = |x: f64| -x.sin() + x.cos();
-        test(var_idx, n_vars, -10000.0..10000.0, &deri, reference);
-        println!("{}", deri);
+        test(var_idx, n_vars, -10000.0..10000.0, deri.clone(), reference);
         let deri = deri.partial(var_idx).unwrap();
         let reference = |x: f64| -x.cos() - x.sin();
-        test(var_idx, n_vars, -10000.0..10000.0, &deri, reference);
-        println!("{}", deri);
+        test(var_idx, n_vars, -10000.0..10000.0, deri.clone(), reference);
         let deri = deri.partial(var_idx).unwrap();
         let reference = |x: f64| x.sin() - x.cos();
-        test(var_idx, n_vars, -10000.0..10000.0, &deri, reference);
+        test(var_idx, n_vars, -10000.0..10000.0, deri.clone(), reference);
 
         let sut = "sin(x)-cos(x)+tan(x)+a";
         println!("{}", sut);
         let var_idx = 1;
         let n_vars = 2;
         let flatex_1 = parse_with_default_ops::<f64>("sin(x)-cos(x)+tan(x)+a").unwrap();
-        let deri = flatex_1.partial(var_idx).unwrap();
         let reference = |x: f64| x.cos() + x.sin() + 1.0 / (x.cos().powf(2.0));
-        test(var_idx, n_vars, -10000.0..10000.0, &deri, reference);
+        test(var_idx, n_vars, -10000.0..10000.0, flatex_1, reference);
 
         let sut = "log(v)*exp(v)+cos(x)+tan(x)+a";
         println!("{}", sut);
         let var_idx = 1;
         let n_vars = 3;
         let flatex = parse_with_default_ops::<f64>(sut).unwrap();
-        let deri = flatex.partial(var_idx).unwrap();
         let reference = |x: f64| 1.0 / x * x.exp() + x.ln() * x.exp();
-        test(var_idx, n_vars, 0.01..100.0, &deri, reference);
+        test(var_idx, n_vars, 0.01..100.0, flatex, reference);
 
         let sut = "a+z+sinh(v)/cosh(v)+b+tanh({v})";
         println!("{}", sut);
         let var_idx = 2;
         let n_vars = 4;
         let flatex = parse_with_default_ops::<f64>(sut).unwrap();
-        let deri = flatex.partial(var_idx).unwrap();
         let reference = |x: f64| {
             (x.cosh() * x.cosh() - x.sinh() * x.sinh()) / x.cosh().powf(2.0)
                 + 1.0 / (x.cosh().powf(2.0))
         };
-        test(var_idx, n_vars, -100.0..100.0, &deri, reference);
+        test(var_idx, n_vars, -100.0..100.0, flatex, reference);
 
         let sut = "w+z+acos(v)+asin(v)+b+atan({v})";
         println!("{}", sut);
         let var_idx = 1;
         let n_vars = 4;
         let flatex = parse_with_default_ops::<f64>(sut).unwrap();
-        let deri = flatex.partial(var_idx).unwrap();
         let reference = |x: f64| {
             1.0 / (1.0 - x.powf(2.0)).sqrt() - 1.0 / (1.0 - x.powf(2.0)).sqrt()
                 + 1.0 / (1.0 + x.powf(2.0))
         };
-        test(var_idx, n_vars, -1.0..1.0, &deri, reference);
+        test(var_idx, n_vars, -1.0..1.0, flatex, reference);
 
         let sut = "sqrt(var)*var^1.57";
         println!("{}", sut);
         let var_idx = 0;
         let n_vars = 1;
         let flatex = parse_with_default_ops::<f64>(sut).unwrap();
-        let deri = flatex.partial(var_idx).unwrap();
         let reference =
             |x: f64| 1.0 / (2.0 * x.sqrt()) * x.powf(1.57) + x.sqrt() * 1.57 * x.powf(0.57);
-        test(var_idx, n_vars, 0.0..100.0, &deri, reference);
+        test(var_idx, n_vars, 0.0..100.0, flatex, reference);
     }
 
     #[test]
@@ -803,12 +851,13 @@ mod tests {
         assert!(eval_str("2*(5+5))").is_err());
     }
 
-    #[cfg(feature="serde_support")]
+    #[cfg(feature = "serde_support")]
     #[test]
     fn test_serde_public_interface() {
         let s = "{x}^(3.0-{y})";
         let flatex = parse_with_default_ops::<f64>(s).unwrap();
         let serialized = serde_json::to_string(&flatex).unwrap();
         let deserialized = serde_json::from_str::<FlatEx<f64>>(serialized.as_str()).unwrap();
-        assert_eq!(s, format!("{}", deserialized));    }
+        assert_eq!(s, format!("{}", deserialized));
+    }
 }
